#version 120

uniform vec4 avlColour;

uniform sampler2D avlBaseTexture;

uniform float avlTextureExists;
uniform float avlUseVertexColour;

uniform float noiseSeedInc;
uniform float brightness;
uniform float saturation;
uniform float contrast;

varying vec2 avlFragBaseTexCoords;
varying vec4 avlFragVertexColour;

/*
** Contrast, saturation, brightness
** Code of this function is from TGM's shader pack
** http://irrlicht.sourceforge.net/phpBB2/viewtopic.php?t=21057
*/

// For all settings: 1.0 = 100% 0.5=50% 1.5 = 150%
vec3 ContrastSaturationBrightness(vec3 color, float brt, float sat, float con)
{
	// Increase or decrease theese values to adjust r, g and b color channels seperately
	const float AvgLumR = 0.5;
	const float AvgLumG = 0.5;
	const float AvgLumB = 0.5;
	
	const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
	
	vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
	vec3 brtColor = color * brt;
	vec3 intensity = vec3(dot(brtColor, LumCoeff));
	vec3 satColor = mix(intensity, brtColor, sat);
	vec3 conColor = mix(AvgLumin, satColor, con);
	return conColor;
}

float rand(vec2 co)
{
	return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

void main()
{
	vec4 b = texture2D(avlBaseTexture, avlFragBaseTexCoords);
	vec4 col = mix(avlColour, avlFragVertexColour, avlUseVertexColour);
	float dist = distance(vec2(0.5, 0.5), avlFragBaseTexCoords*1.0);
	vec4 c = mix(col, col * b * ((rand(avlFragBaseTexCoords+noiseSeedInc)*0.1f)+0.9f), avlTextureExists);
	
	//gl_FragColor = mix(vec4(redFlash, 0.0, 0.0, 1.0), c, (1.0 - (dist * dist)));
	//c.rgb = ContrastSaturationBrightness(c.rgb, 1.0, 1.0+(7.0*redFlash), 1.0+(3.0*redFlash));
	c.rgb = ContrastSaturationBrightness(c.rgb, 1.0+(2.0*brightness), 1.0+(7.0*saturation), 1.0+(3.0*contrast));
	c = mix(vec4(brightness, 0.0, 0.0, 1.0), c, (1.0 - (dist * dist)));

	gl_FragColor = c;
}
