//	Apple.h - Apple for the snakes to eat.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef APPLE_H_
#define APPLE_H_

#include "ThreeFloats.h"
#include "Star.h"
#include "NPC.h"

class Drawer;

///	Apple for the snakes to eat.
class Apple : public NPC
{
  public:
	///	Constructor.
	Apple(const TwoFloats& pos);
	///	Destructor.
	~Apple();

	///	Moves the NPC.
	void update(const TwoFloats& drawSize) {};
	///	Draws the NPC.
	void draw(shared_ptr<Drawer>& d);
	
	///	Kills the NPC.
	void setDead();
	///	Returns true if the NPC can be deleted.
	bool isDead() const {return dead;};

	///	Updates the collidable's position.
	void setPosition(const TwoFloats& pos);
	///	Returns the collidable's position.
	const TwoFloats& getPosition() const {return position;};

	///	Updates the collidable's direction.
	void setDirection(const TwoFloats& dir) {};
	///	Returns the collidable's direction.
	const TwoFloats& getDirection() const {return direction;};

	///	Returns the collidable's radius.
	float getRadius() const {return 16.0f;};

	///	Returns the collidable's name.
	const std::wstring& getName() const {return name;};

	///	Called to test/react to collisions with other objects.
	void collide(const shared_ptr<Collidable>& other) {};

	///	Sets the apple's colour.
	void setColour(const ThreeFloats& col);
	///	Sets the background star's colour.
	void setBackColour(const ThreeFloats& col);

	///	Returns the apple's colour.
	const ThreeFloats& getColour() const {return colour;};
  private:
	///	The apple's position.
	TwoFloats position;
	///	The apple's direction.
	TwoFloats direction;
	///	The apple's name.
	std::wstring name;
	///	The apple's colour.
	ThreeFloats colour;

	///	The background star.
	Star star;
	///	The star's colour.
	ThreeFloats backColour;

	///	True if the apple's now dead.
	bool dead;
};

#endif
