//	Ball.h - The ball.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BALL_H_
#define BALL_H_

#include "Collidable.h"

class ParticleGenerator;
class MinimalSounder;
class GamePlayState;
class Drawer;

///	The ball.
class Ball : public Collidable
{
  public:
	///	Constructor.
	Ball(const TwoFloats& pos,
		 bool dir,
		 shared_ptr<MinimalSounder>& sound,
		 ParticleGenerator& party,
		 GamePlayState& state);
	///	Destructor.
	~Ball();

	///	Moves the ball.
	void update(const TwoFloats& drawSize);
	///	Draws the ball.
	void draw(shared_ptr<Drawer>& d);

	///	Kicks the ball in the current direction.
	void kick();
	///	Resets the ball's speed.
	void resetSpeed(bool dir);

	///	Sets the ball's owner (by the owner's colour).
	void setOwner(const ThreeFloats& col);
	///	Returns the ball's owner (black if no owner).
	const ThreeFloats& getOwner() const {return owner;};
	
	///	Updates the collidable's position.
	void setPosition(const TwoFloats& pos);
	///	Returns the collidable's position.
	const TwoFloats& getPosition() const {return position;};

	///	Updates the collidable's direction.
	void setDirection(const TwoFloats& dir);
	///	Returns the collidable's direction.
	const TwoFloats& getDirection() const {return direction;};

	///	Returns the collidable's radius.
	float getRadius() const {return 16.0f;};

	///	Returns the collidable's name.
	const std::wstring& getName() const {return name;};

	///	Called to test/react to collisions with other objects.
	void collide(const shared_ptr<Collidable>& other);
  private:
	///	The ball's position.
	TwoFloats position;
	///	The ball's name for getName().
	std::wstring name;

	///	The ball's direction.
	TwoFloats direction;
	///	The ball's speed.
	float speed;

	///	The ball's owner.
	ThreeFloats owner;

	///	Our copy of the sounder.
	shared_ptr<MinimalSounder>& sounder;
	///	Our copy of the particle generator.
	ParticleGenerator& particles;
	///	Our copy of the gameplay state.
	GamePlayState& gameState;
};

#endif
