//	Bullet.h - Bullet for players to shoot.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BULLET_H_
#define BULLET_H_

#include "ThreeFloats.h"
#include "NPC.h"

class Drawer;

///	Bullet for players to shoot.
class Bullet : public NPC
{
  public:
	///	Constructor.
	Bullet(const TwoFloats& pos,
		   const ThreeFloats& col,
		   const TwoFloats& dir);
	///	Destructor.
	~Bullet();

	///	Moves the NPC.
	void update(const TwoFloats& drawSize);
	///	Draws the NPC.
	void draw(shared_ptr<Drawer>& d);
	
	///	Kills the NPC.
	void setDead();
	///	Returns true if the NPC can be deleted.
	bool isDead() const {return dead;};

	///	Updates the collidable's position.
	void setPosition(const TwoFloats& pos);
	///	Returns the collidable's position.
	const TwoFloats& getPosition() const {return position;};

	///	Updates the collidable's direction.
	void setDirection(const TwoFloats& dir) {};
	///	Returns the collidable's direction.
	const TwoFloats& getDirection() const {return direction;};

	///	Returns the collidable's radius.
	float getRadius() const {return 12.0f;};

	///	Returns the collidable's name.
	const std::wstring& getName() const {return name;};

	///	Called to test/react to collisions with other objects.
	void collide(const shared_ptr<Collidable>& other) {};

	///	Sets the Bullet's colour.
	void setColour(const ThreeFloats& col);
	///	Returns the Bullet's colour.
	const ThreeFloats& getColour() const {return colour;};
  private:
	///	The Bullet's position.
	TwoFloats position;
	///	The Bullet's direction.
	TwoFloats direction;
	///	The Bullet's name.
	std::wstring name;
	///	The Bullet's colour.
	ThreeFloats colour;

	///	True if the Bullet's now dead.
	bool dead;
};

#endif
