//	CaptureTheFlagPlayer.cpp - The CaptureTheFlag player.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "CaptureTheFlagPlayer.h"
#include "drawing/Drawer.h"
#include "MiscHelper.h"
#include "Zombie.h"
#include "Bullet.h"
#include "Flag.h"

//------------------------------------------------------------------------------
CaptureTheFlagPlayer::CaptureTheFlagPlayer(const TwoFloats& pos, const ThreeFloats& col):
Player(pos),
position(pos.x, pos.y-64.0f),
name(L"CaptureTheFlagPlayer"),
colour(col),
stunned(0),
angle(0.0f),
armPos(0.0f),
upIsDown(false),
downIsDown(false),
leftIsDown(false),
rightIsDown(false),
dead(false)
{
	
}

//------------------------------------------------------------------------------
CaptureTheFlagPlayer::~CaptureTheFlagPlayer()
{

}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::update(const TwoFloats& drawSize)
{
	const float speed(8.0f);

	if(stunned > 0)
	{
		--stunned;

		star.update();

		return;
	}

	position += direction * speed;
	if(position.x < 16.0f)
		position.x = 16.0f;
	else if(position.x > (drawSize.x-16.0f))
		position.x = drawSize.x-16.0f;
	if(position.y < (32.0f+16.0f))
		position.y = (32.0f+16.0f);
	else if(position.y > (drawSize.y-(32.0f+16.0f)))
		position.y = drawSize.y-(32.0f+16.0f);

	if(flag)
		flag->setPosition(position-16.0f);

	if(direction != 0.0f)
	{
		armPos += 0.033f;
		if(armPos > 1.0f)
			armPos -= 1.0f;
	}
	else if(armPos > 0.0f)
	{
		armPos += 0.1f;
		if(armPos > 1.0f)
			armPos = 0.0f;
	}
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::draw(shared_ptr<Drawer>& d)
{
	TwoFloats pivot;
	TwoFloats tempPos;
	TwoFloats tempDir(sinf(angle), -cosf(angle));
	TwoFloats perpVector(tempDir.perpendicular());
	float sinArm = sinf(armPos * 2.0f * NIALL_PI);

	if(stunned > 0)
		star.draw(d, colour*0.66f);

	d->translate(position);
	d->rotate(angle + NIALL_PI, 0.0f);

	//Draw right arm.
	pivot.x = 24.0f;
	pivot.y = 24.0f;
	tempPos.x = -24.0f;
	tempPos.y = -pivot.y + (sinArm * 4.0f);
	d->drawImage(L"PersonRightArm", tempPos, 1.0f, colour);

	//Draw left arm.
	pivot.x = 24.0f;
	pivot.y = 24.0f;
	tempPos.x = -24.0f;
	tempPos.y = -pivot.y - (sinArm * 4.0f);
	d->drawImage(L"PersonLeftArm", tempPos, 1.0f, colour);

	//Draw body.
	pivot.x = 24.0f;
	pivot.y = 24.0f;
	tempPos.x = -24.0f;
	tempPos.y = -24.0f;
	d->drawImage(L"PersonHead", tempPos, 1.0f, colour);

	d->rotate(-(angle+NIALL_PI), 0.0f);
	d->translate(TwoFloats(-position.x, -position.y));
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::keyDown(const std::wstring& action)
{
	if(action == L"Up")
	{
		upIsDown = true;
		if(!downIsDown)
			direction.y = -1.0f;
		else
			direction.y = 0.0f;
	}
	else if(action == L"Down")
	{
		downIsDown = true;
		if(!upIsDown)
			direction.y = 1.0f;
		else
			direction.y = 0.0f;
	}
	else if(action == L"Left")
	{
		leftIsDown = true;
		if(!rightIsDown)
			direction.x = -1.0f;
		else
			direction.x = 0.0f;
	}
	else if(action == L"Right")
	{
		rightIsDown = true;
		if(!leftIsDown)
			direction.x = 1.0f;
		else
			direction.x = 0.0f;
	}
	direction = direction.normal();
	
	if(direction != 0.0f)
	{
		TwoFloats angleDir(-direction.x, direction.y);

		angle = angleDir.getAngle();

		realDirection = direction;
	}
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::keyUp(const std::wstring& action)
{
	if(action == L"Up")
	{
		upIsDown = false;
		if(!downIsDown)
			direction.y = 0.0f;
		else
			direction.y = 1.0f;
	}
	else if(action == L"Down")
	{
		downIsDown = false;
		if(!upIsDown)
			direction.y = 0.0f;
		else
			direction.y = -1.0f;
	}
	else if(action == L"Left")
	{
		leftIsDown = false;
		if(!rightIsDown)
			direction.x = 0.0f;
		else
			direction.x = 1.0f;
	}
	else if(action == L"Right")
	{
		rightIsDown = false;
		if(!leftIsDown)
			direction.x = 0.0f;
		else
			direction.x = -1.0f;
	}
	direction = direction.normal();
	
	if(direction != 0.0f)
	{
		TwoFloats angleDir(-direction.x, direction.y);

		angle = angleDir.getAngle();
		realDirection = direction;
	}
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::joystick(const std::wstring& action, float val)
{
	if((action == L"Up") || (action == L"Down"))
	{
		direction.y = val;
		if(fabsf(direction.y) < 0.15f)
			direction.y = 0.0f;
	}
	else if((action == L"Left") || (action == L"Right"))
	{
		direction.x = val;
		if(fabsf(direction.x) < 0.15f)
			direction.x = 0.0f;
	}

	if(direction != 0.0f)
	{
		TwoFloats angleDir(-direction.x, direction.y);

		angle = angleDir.getAngle();
		realDirection = direction.normal();
	}
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::setPosition(const TwoFloats& pos)
{
	position = pos;
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::setDirection(const TwoFloats& dir)
{
	direction = dir;
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::collide(const shared_ptr<Collidable>& other)
{
	const float radius(getRadius());
	TwoFloats otherPos(other->getPosition());
	const float otherRad(other->getRadius());
	TwoFloats otherDir(other->getDirection());
	const std::wstring& otherName(other->getName());

	if((otherName == L"Flag"))
	{
		if(!stunned)
		{
			if(otherPos.getDistance(position) < (otherRad+radius))
			{
				shared_ptr<Flag> f(dynamic_pointer_cast<Flag>(other));

				if(f->getColour() != colour)
					flag = other;
				else 
					f->reset();
			}
		}
	}
	else if(otherName == L"Bullet")
	{
		if(otherPos.getDistance(position) < (otherRad+radius))
		{
			shared_ptr<Bullet> bullet(dynamic_pointer_cast<Bullet>(other));

			if(bullet->getColour() != colour)
			{
				bullet->setDead();
				if(flag)
				{
					shared_ptr<Flag> f(dynamic_pointer_cast<Flag>(flag));

					f->flyOff();
					flag = 0;

					stunned = 60;
					star.setPosition(position, -0.01f);
				}
			}
		}
	}
	else if(otherName == L"Zombie")
	{
		if(otherPos.getDistance(position) < (otherRad+radius))
		{
			shared_ptr<Zombie> z(dynamic_pointer_cast<Zombie>(other));

			if(z)
			{
				if(z->getColour() != colour)
					dead = true;
				else
				{
					float offset;
					TwoFloats diff(position - otherPos);
					float dist = diff.getMagnitude();
					float rad = getRadius() + other->getRadius();

					diff = diff.normal();
					offset = (rad - dist);
					otherPos -= diff * offset;

					other->setPosition(otherPos);
				}
			}
		}
	}
	else if((otherName != L"PongPlayer") && (otherName != L"Apple"))
	{
		if(otherPos.getDistance(position) < (otherRad+radius))
		{
			float offset;
			TwoFloats diff(position - otherPos);
			float dist = diff.getMagnitude();
			float rad = getRadius() + other->getRadius();

			diff = diff.normal();
			offset = (rad - dist);
			otherPos -= diff * offset;

			other->setPosition(otherPos);
		}
	}
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::setColour(const ThreeFloats& col)
{
	colour = col;
}

//------------------------------------------------------------------------------
void CaptureTheFlagPlayer::setFlag(shared_ptr<Flag>& f)
{
	flag = f;
}
