//	Collidable.h - Base class for any collidable objects.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef COLLIDABLE_H_
#define COLLIDABLE_H_

#include "c0xHeaders.h"
#include "TwoFloats.h"

#include <string>

///	Base class for any collidable objects.
class Collidable
{
  public:
	///	Constructor.
	Collidable() {};
	///	Destructor.
	virtual ~Collidable() {};

	///	Updates the collidable's position.
	virtual void setPosition(const TwoFloats& pos) = 0;
	///	Returns the collidable's position.
	virtual const TwoFloats& getPosition() const = 0;

	///	Updates the collidable's direction.
	virtual void setDirection(const TwoFloats& dir) = 0;
	///	Returns the collidable's direction.
	virtual const TwoFloats& getDirection() const = 0;

	///	Returns the collidable's radius.
	virtual float getRadius() const = 0;

	///	Returns the collidable's name.
	virtual const std::wstring& getName() const = 0;

	///	Called to test/react to collisions with other objects.
	virtual void collide(const shared_ptr<Collidable>& other) = 0;
};

#endif
