//	DeadPlayer.h - Gravestone for player killed by zombies.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef DEADPLAYER_H_
#define DEADPLAYER_H_

#include "ThreeFloats.h"
#include "Player.h"

///	Gravestone for player killed by zombies.
class DeadPlayer : public Player
{
  public:
	///	Constructor.
	DeadPlayer(const TwoFloats& pos, const ThreeFloats& col);
	///	Destructor.
	~DeadPlayer();

	///	Moves the player.
	void update(const TwoFloats& drawSize) {};
	///	Draws the player.
	void draw(shared_ptr<Drawer>& d);

	///	Called when a key is pressed down.
	void keyDown(const std::wstring& action) {};
	///	Called when a key is pressed up.
	void keyUp(const std::wstring& action) {};
	///	Called when a joystick event is received.
	void joystick(const std::wstring& action, float val) {};
	
	///	Updates the collidable's position.
	void setPosition(const TwoFloats& pos) {};
	///	Returns the collidable's position.
	const TwoFloats& getPosition() const {return position;};

	///	Updates the collidable's direction.
	void setDirection(const TwoFloats& dir) {};
	///	Returns the collidable's direction.
	const TwoFloats& getDirection() const {return direction;};

	///	Returns the collidable's radius.
	float getRadius() const {return 24.0f;};

	///	Returns the collidable's name.
	const std::wstring& getName() const {return name;};

	///	Called to test/react to collisions with other objects.
	void collide(const shared_ptr<Collidable>& other);

	///	Sets the bat's colour.
	void setColour(const ThreeFloats& col) {};
	///	Returns the player's colour.
	const ThreeFloats& getColour() const {return colour;};
  private:
	///	The bat's position.
	TwoFloats position;
	///	For returning from getName().
	std::wstring name;
	///	The bat's colour.
	ThreeFloats colour;

	///	The direction the bat is currently moving.
	TwoFloats direction;
};

#endif
