//	Flag.cpp - Flag for the capture the flag game.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "Random.h"
#include "Flag.h"

//------------------------------------------------------------------------------
Flag::Flag(const TwoFloats& pos, const ThreeFloats& col):
position(pos),
name(L"Flag"),
colour(col),
startPosition(pos),
fling(0.0f),
dead(false)
{
	position.x -= 4.0f;
}

//------------------------------------------------------------------------------
Flag::~Flag()
{
	
}

//------------------------------------------------------------------------------
void Flag::update(const TwoFloats& drawSize)
{
	const float speed(16.0f);

	if(fling > 0.0f)
	{
		position += direction * fling * speed;

		fling -= 0.05f;
	}
}

//------------------------------------------------------------------------------
void Flag::draw(shared_ptr<Drawer>& d)
{
	TwoFloats tempPos(position-16.0f);

	//Offset the image so the flagpole is centred.
	tempPos.x += 9.0f;
	d->drawImage(L"FlagBack", tempPos-2.0f);
	d->drawImage(L"Flag", tempPos, 1.0f, colour);
}

//------------------------------------------------------------------------------
void Flag::reset()
{
	setPosition(startPosition);
}

//------------------------------------------------------------------------------
void Flag::flyOff()
{
	direction.x = Random::getGlobal().getRandomFloat();
	direction.y = Random::getGlobal().getRandomFloat();
	direction = direction.normal();

	fling = 1.0f;
}

//------------------------------------------------------------------------------
void Flag::setDead()
{
	dead = true;
}

//------------------------------------------------------------------------------
void Flag::setPosition(const TwoFloats& pos)
{
	position = pos;
	fling = 0.0f;
}

//------------------------------------------------------------------------------
void Flag::setColour(const ThreeFloats& col)
{
	colour = col;
}
