//	Star.cpp - Simple winner's star class.
//  ----------------------------------------------------------------------------
//	This file is part of 'NiallsAVLib', base code for any kind of audiovisual
//	apps.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "MiscHelper.h"
#include "Star.h"

//------------------------------------------------------------------------------
Star::Star():
rotation(0.0f),
rotDir(0.01f)
{
	scale[0] = 0.0f;
	scale[1] = NIALL_PI;
}

//------------------------------------------------------------------------------
Star::~Star()
{

}

//------------------------------------------------------------------------------
void Star::update()
{
	rotation += rotDir;
	if(rotation > (2.0f*NIALL_PI))
		rotation -= 2.0f*NIALL_PI;

	scale[0] += 0.02f;
	if(scale[0] > (2.0f*NIALL_PI))
		scale[0] -= 2.0f*NIALL_PI;

	scale[1] += 0.005f;
	if(scale[1] > (2.0f*NIALL_PI))
		scale[1] -= 2.0f*NIALL_PI;
}

//------------------------------------------------------------------------------
void Star::draw(shared_ptr<Drawer>& d, const ThreeFloats& col)
{
	float tempf;

	tempf = 0.75f + ((sinf(scale[0])+1.0f) * 0.125f);
	d->drawRotImage(L"Star0",
					position-(64.0f*tempf),
					rotation,
					64.0f,
					tempf,
					col);

	tempf = 0.75f + ((cosf(scale[0])+1.0f) * 0.125f);
	d->drawRotImage(L"Star1",
					position-(64.0f*tempf),
					rotation,
					64.0f,
					tempf,
					col);
}

//------------------------------------------------------------------------------
void Star::setPosition(const TwoFloats& pos, float rot)
{
	position = pos;
	rotDir = rot;
}
