//	Zombie.cpp - A zombie.
//  ----------------------------------------------------------------------------
//	This file is part of 'Minimal2PGame', a Ludum Dare entry.
//	Copyright (C) 2013  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "drawing/Drawer.h"
#include "GameCounter.h"
#include "Bullet.h"
#include "Player.h"
#include "Zombie.h"
#include "Random.h"

//------------------------------------------------------------------------------
Zombie::Zombie(const TwoFloats& pos,
			   const ThreeFloats& col,
			   shared_ptr<Player>& p,
			   GameCounter& counter):
position(pos),
name(L"Zombie"),
colour(col),
origColour(col),
speed(1.0f + (Random::getGlobal().getRandomFloat() * 4.0f)),
player(p),
dead(false),
gameCounter(counter)
{
	direction.y = 0.0f;
	if(position.x < 640.0f)
		direction.x = 1.0f;
	else
		direction = -1.0f;
	realDir = direction;

	colour *= 0.66f + (Random::getGlobal().getRandomFloat() * 0.25f);
}

//------------------------------------------------------------------------------
Zombie::~Zombie()
{

}

//------------------------------------------------------------------------------
void Zombie::update(const TwoFloats& drawSize)
{
	direction = (player->getPosition() - position).normal();

	if(realDir != direction)
		realDir += (direction-realDir) * 0.05f;

	position += realDir * speed;
}

//------------------------------------------------------------------------------
void Zombie::draw(shared_ptr<Drawer>& d)
{
	TwoFloats tempDir(realDir.x, -realDir.y);

	d->drawRotImage(L"Zombie",
					position-8.0f,
					tempDir.getAngle(),
					8.0f,
					1.0f,
					colour);
}

//------------------------------------------------------------------------------
void Zombie::setDead()
{
	dead = true;
}

//------------------------------------------------------------------------------
void Zombie::setPosition(const TwoFloats& pos)
{
	position = pos;
}

//------------------------------------------------------------------------------
void Zombie::collide(const shared_ptr<Collidable>& other)
{
	if(other->getName() == L"Ball")
	{
		if(other->getPosition().getDistance(position) < (other->getRadius()+getRadius()))
			dead = true;
	}
	else if(other->getName() == L"Bullet")
	{
		if(other->getPosition().getDistance(position) < (other->getRadius()+getRadius()))
		{
			shared_ptr<Bullet> b(dynamic_pointer_cast<Bullet>(other));

			if(b)
			{
				dead = true;
				b->setDead();

				if(b->getColour() != origColour)
				{
					if(origColour == ThreeFloats(1.0f, 0.45f, 0.45f))
						gameCounter.incrementCount(L"RightScore");
					else
						gameCounter.incrementCount(L"LeftScore");
				}
				else
				{
					if(origColour == ThreeFloats(1.0f, 0.45f, 0.45f))
						gameCounter.incrementCount(L"LeftScore", -1);
					else
						gameCounter.incrementCount(L"RightScore", -1);
				}
			}
		}
	}
}

//------------------------------------------------------------------------------
void Zombie::setColour(const ThreeFloats& col)
{
	colour = col;
}
